<?php
/**
 * @package    db8optionslite
 *
 * @author     Peter Martin <joomla@db8.eu>
 * @copyright  Copyright 2016-2017 by Peter Martin
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://db8.eu
 */

defined('_JEXEC') or die;

/**
 * Db8optionslite Controller.
 *
 * @package  db8optionslite
 * @since    1.0
 */
class Db8optionsliteControllerComponents extends JControllerAdmin
{
	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return  JModelLegacy|boolean  Model object on success; otherwise false on failure.
	 *
	 * @since   0.1
	 */
	public function getModel($name = 'Components', $prefix = 'Db8optionsliteModel', $config = array('ignore_request' => true))
	{
		return parent::getModel($name, $prefix, $config);
	}

	/**
	 * Method to generate a backup of the current parameters of an extension
	 *
	 * @since   0.1
	 */
	public function backup()
	{
		$ids = $this->input->get('cid', array(), 'array');

		if (empty($ids))
		{
			JError::raiseWarning(500, JText::_('JERROR_NO_ITEMS_SELECTED'));
		}
		else
		{
			$model = $this->getModel();

			if (!$names = $model->backup($ids))
			{
				JError::raiseWarning(500, $model->getError());
			}

			$message = "Backuped:<br>";
			$message .= "<ul>";
			foreach ($names as $name)
				$message .= "<li>$name</li>";
			$message .= "</ul>";
		}

		$this->setRedirect(JRoute::_('index.php?option=com_db8optionslite&view=components', false), $message);
	}


	public function batch()
	{
		//Db8optionsliteViewLiteversion
		$view = $this->getView( 'Liteversion', 'html' );
		$view->setLayout('default');
		$view->display();
	}

	/**
	 * Method to change the parameters with imported or previous parameters
	 *
	 * @since   0.1
	 */
	public function change()
	{
		$input = JFactory::getApplication()->input;
		$id    = $input->get('cid', array(), 'array');
		$id    = $id[0];

		$type = $input->getRaw('type_of_param');

		$db = JFactory::getDbo();

		$query = $db->getQuery(true);
		$query->select('concat_ws(".", name, type, element, folder, client_id)')
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('extension_id') . ' = ' . $db->quote($id));
		$db->setQuery($query);
		$extension_key = $db->loadResult();

		if ($type == 'imported')
		{
			$subQuery = $db->getQuery(true);
			$subQuery->select($db->quoteName('params_imported'))
				->from($db->quoteName('#__db8optionslite'))
				->where($db->quoteName('extension_key') . ' = ' . $db->quote($extension_key));

			$query = $db->getQuery(true);
			$query->update($db->quoteName('#__extensions'))
				->set($db->quoteName('params') . ' = ( ' . $subQuery . ' )')
				->where($db->quoteName('extension_id') . ' = ' . $db->quote($id));

			$db->setQuery($query);
			$db->execute();
		}
		if ($type == 'original')
		{
			$subQuery = $db->getQuery(true);
			$subQuery->select($db->quoteName('params_previous'))
				->from($db->quoteName('#__db8optionslite'))
				->where($db->quoteName('extension_key') . ' = ' . $db->quote($extension_key));

			$query = $db->getQuery(true);
			$query->update($db->quoteName('#__extensions'))
				->set($db->quoteName('params') . ' = ( ' . $subQuery . ' )')
				->where($db->quoteName('extension_id') . ' = ' . $db->quote($id));

			$db->setQuery($query);
			$db->execute();
		}

		$this->setRedirect(JRoute::_('index.php?option=com_db8optionslite&view=components', false));
	}

	/**
	 * Method to import a JSON text file into the parameters_imported field of an extension
	 *
	 * @since   0.1
	 */
	public function import()
	{
        jimport('joomla.filesystem.file');
        jimport('joomla.filesystem.folder');

        $imported_extensions = array();

		$input = JFactory::getApplication()->input;

        $multiple = $input->files->get('importjson');
        if ($multiple[0]['name']) {
            foreach ($multiple as $json_or_zip) {
                $ext = pathinfo($json_or_zip['name'], PATHINFO_EXTENSION);

                if ($ext == 'json') {
                    $imported_extensions[] = $this->importhelper($json_or_zip["tmp_name"]);
                }
            }
        } else {
            $id    = $input->get('cid', array(), 'array');
            $id    = $id[0];

            $file = $input->files->get('importfile' . $id);
            $imported_extensions[] = $this->importhelper($file["tmp_name"]);
        }

        $message = '';
        foreach ($imported_extensions as $imported_extension) {
            if ($imported_extension['status'])
                $message .= "Successfully Imported: " . $imported_extension['name'] . '<br>';
            else
                $message .= "Unsuccessfully Imported: " . $imported_extension['name'] . '<br>';
        }

		$this->setRedirect(JRoute::_('index.php?option=com_db8optionslite&view=components', false), $message);
	}

	protected function importhelper($filename) {

        $model = $this->getModel();

        $file_rows     = file($filename);
        $extension_key = trim(substr($file_rows[1], strpos($file_rows[1], ":") + 1));

        $json = $file_rows[2];

        return $model->import($extension_key, $json);
    }

	/**
	 * Method to export the parameters field of an extension to a JSON text file
	 *
	 * @since   0.1
	 */
	public function export()
	{
		// Check for request forgeries
		//JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));

		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');

		$user = JFactory::getUser();
		$ids  = $this->input->get('cid', array(), 'array');

        $is_zip = false;
        if (count($ids) > 1) $is_zip = true;

		$app = JFactory::getApplication();
		$db  = JFactory::getDbo();

		$query = $db->getQuery(true);

		$query->select('extension_id, type, folder, element, params, name, client_id')
			->from('#__extensions')
			->where('extension_id IN (' . implode(',', $ids) . ')');

		$db->setQuery($query);
		$results = $db->loadObjectList();
		foreach ($results as $result)
		{

			if ($result->folder === "")
			{
				$result->folder = "no";
			}
			$result->filename = $result->type . '-' . $result->folder . '-' . $result->element . '-' .
				date("Ymd") . '-' . date("His") . '.json';
		}

		if (empty($ids))
		{
			JError::raiseWarning(500, JText::_('JERROR_NO_ITEMS_SELECTED'));
		}

		$version = $this->getExtensionVersion('com_db8optionslite');

		if (JFolder::exists(JPATH_ADMINISTRATOR . "/components/com_db8optionslite/export"))
			JFolder::delete(JPATH_ADMINISTRATOR . "/components/com_db8optionslite/export");

		$filesToZip = array();
        $file_name = '';
		foreach ($results as $result)
		{
			$extension_key = $result->name . '.' . $result->type . '.' . $result->element . '.' . $result->folder . '.' . $result->client_id;
			$file_name     = str_replace(".", "-", $extension_key) . "-" . date("Ymd") . '-' . date("His") . '.json';
            $file_path     = JPATH_ADMINISTRATOR . "/components/com_db8optionslite/export/" . $file_name;
			$file_content  = "Created by db8optionslite v" . $version . " on " . date("Y/m/d") . " " . date("H:i:s");
			$file_content .= "\r\nExtension key: " . $extension_key;
			$file_content .= "\r\n" . $result->params;

			JFile::write($file_path, $file_content);
			$filesToZip[] = array('name' => $file_name, 'data' => $file_content);
		}

		@ob_clean();

		header("Pragma: public");
		header("Expires: 0"); // set expiration time
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");

		header("Content-Type: application/force-download");
		header("Content-Type: application/octet-stream");
		header("Content-Type: application/download");
		if ($is_zip) header("Content-Type: application/zip");

		header("Content-Disposition: attachment; filename=" . $file_name . ";");

		header("Content-Transfer-Encoding: binary");
		header('Content-Length: ' . filesize($file_path));

		readfile($file_path);

		exit;
	}

	/**
	 * Method to get the version number of an extension.
	 *
	 * @param   string  $ext    The name of the extension.
	 *
	 * @return  mixed
	 *
	 * @since   0.1
	 */
	function getExtensionVersion($ext)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('manifest_cache')
			->from($db->quoteName('#__extensions'))
			->where('element = ' . $db->quote($ext));
		$db->setQuery($query);

		$manifest = json_decode($db->loadResult(), true);

		return $manifest['version'];
	}
}
