<?php
/**
 * @package    db8optionslite
 *
 * @author     Peter Martin <joomla@db8.eu>
 * @copyright  Copyright 2016-2017 by Peter Martin
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://db8.eu
 */

defined('_JEXEC') or die;

/**
 * Db8optionslite helper.
 *
 * @package     A package name
 * @since       1.0
 */
class Db8optionsliteHelper
{
	/**
	 * Render submenu.
	 *
	 * @param   string  $vName  The name of the current view.
	 *
	 * @return  void.
	 *
	 * @since   1.0
	 */
	public static function addSubmenu($vName)
	{

		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_DASHBOARD'),
			'index.php?option=com_db8optionslite&view=dashboard',
			$vName == 'dashboard');

		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_COMPONENTS'),
			'index.php?option=com_db8optionslite&view=components',
			$vName == 'components');

		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_PLUGINS'),
			'index.php?option=com_db8optionslite&view=liteversion',
			$vName == 'plugins');

        CustomSidebar::addEntry(
            JText::_('COM_DB8OPTIONSLITE_MENU_MODULES'),
            'index.php?option=com_db8optionslite&view=liteversion',
            $vName == 'modules');

        CustomSidebar::addEntry(
            JText::_('COM_DB8OPTIONSLITE_MENU_TEMPLATES'),
            'index.php?option=com_db8optionslite&view=liteversion',
            $vName == 'templates');

/*		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_OPTIONS'),
			'index.php?option=com_db8optionslite&view=options',
			$vName == 'options');*/

		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_MENUITEMS'),
			'index.php?option=com_db8optionslite&view=menuitems',
			$vName == 'menuitems');

		CustomSidebar::addEntry(
			JText::_('COM_DB8OPTIONSLITE_MENU_ABOUT'),
			'index.php?option=com_db8optionslite&view=about',
			$vName == 'about');
	}

	public static function getExtensionTypes()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select('DISTINCT type')
			->from('#__extensions');
		$db->setQuery($query);
		$types = $db->loadColumn();

		$options = array();

		foreach ($types as $type)
		{
			$options[] = JHtml::_('select.option', $type, JText::_('COM_DB8OPTIONSLITE_FILTER_TYPE_' . strtoupper($type)));
		}

		return $options;
	}

	public static function getExtensionFolders()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select('DISTINCT folder')
			->from('#__extensions')
			->where('folder <>""');
		$db->setQuery($query);
		$folders = $db->loadColumn();

		$options = array();

		foreach ($folders as $folder)
		{
			$options[] = JHtml::_('select.option', $folder, $folder);
		}

		return $options;
	}

	public static function getClientOptions()
	{
		// Build the filter options.
		$options   = array();
		$options[] = JHtml::_('select.option', '0', JText::_('JSITE'));
		$options[] = JHtml::_('select.option', '1', JText::_('JADMINISTRATOR'));

		return $options;
	}


	/*
	 * 	type
		folder
		element
		location
		client_id
	 */

	public static function getConfigUrl($name, $type, $element, $folder, $client_id, $id)
	{
		switch ($type)
		{
			case 'component':
				$configUrl = 'index.php?option=com_config&view=component&component=' . $element . '&path=&return=';
				break;
			case 'module':
				//$configUrl = 'index.php?option=com_modules&view=module&layout=edit&id=' . $id;
				$configUrl = '#';
				break;
			case 'plugin':
				//$configUrl = 'index.php?option=com_plugins&view=plugin&layout=edit&extension_id=' . $id;
				$configUrl = '#';
				break;
			case 'template':
				//$configUrl = 'index.php?option=com_templates&view=style&layout=edit&id=' . $id;
				$configUrl = '#';
				break;
			case 'language':
				//$configUrl = 'index.php?option=com_languages&view=language&layout=edit&lang_id=' . $id;
				$configUrl = '#';
				break;
			case 'file':
				$configUrl = '#';
				break;
			case 'library':
				$configUrl = '#';
				break;
			case 'package':
				$configUrl = '#';
				break;
		}

		return $configUrl;
	}


	public static function getExtensionGroupes()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select('DISTINCT folder')
			->from('#__extensions')
			->where('folder != ' . $db->quote(''))
			->order('folder');
		$db->setQuery($query);
		$folders = $db->loadColumn();

		$options = array();

		foreach ($folders as $folder)
		{
			$options[] = JHtml::_('select.option', $folder, $folder);
		}

		return $options;
	}

	/**
	 * Returns a published state on a grid
	 *
	 * @param   integer  $value     The state value.
	 * @param   integer  $i         The row index
	 * @param   boolean  $enabled   An optional setting for access control on the action.
	 * @param   string   $checkbox  An optional prefix for checkboxes.
	 *
	 * @return  string        The Html code
	 *
	 * @see JHtmlJGrid::state
	 *
	 * @since   1.7.1
	 */
	public static function state($value, $i, $enabled = true, $checkbox = 'cb')
	{
		$states = array(
			9 => array(
				'unpublish',
				'',
				'COM_MENUS_HTML_UNPUBLISH_HEADING',
				'',
				true,
				'publish',
				'publish'
			),
			8 => array(
				'publish',
				'',
				'COM_MENUS_HTML_PUBLISH_HEADING',
				'',
				true,
				'unpublish',
				'unpublish'
			),
			7 => array(
				'unpublish',
				'',
				'COM_MENUS_HTML_UNPUBLISH_SEPARATOR',
				'',
				true,
				'publish',
				'publish'
			),
			6 => array(
				'publish',
				'',
				'COM_MENUS_HTML_PUBLISH_SEPARATOR',
				'',
				true,
				'unpublish',
				'unpublish'
			),
			5 => array(
				'unpublish',
				'',
				'COM_MENUS_HTML_UNPUBLISH_ALIAS',
				'',
				true,
				'publish',
				'publish'
			),
			4 => array(
				'publish',
				'',
				'COM_MENUS_HTML_PUBLISH_ALIAS',
				'',
				true,
				'unpublish',
				'unpublish'
			),
			3 => array(
				'unpublish',
				'',
				'COM_MENUS_HTML_UNPUBLISH_URL',
				'',
				true,
				'publish',
				'publish'
			),
			2 => array(
				'publish',
				'',
				'COM_MENUS_HTML_PUBLISH_URL',
				'',
				true,
				'unpublish',
				'unpublish'
			),
			1 => array(
				'unpublish',
				'COM_MENUS_EXTENSION_PUBLISHED_ENABLED',
				'COM_MENUS_HTML_UNPUBLISH_ENABLED',
				'COM_MENUS_EXTENSION_PUBLISHED_ENABLED',
				true,
				'publish',
				'publish'
			),
			0 => array(
				'publish',
				'COM_MENUS_EXTENSION_UNPUBLISHED_ENABLED',
				'COM_MENUS_HTML_PUBLISH_ENABLED',
				'COM_MENUS_EXTENSION_UNPUBLISHED_ENABLED',
				true,
				'unpublish',
				'unpublish'
			),
			-1 => array(
				'unpublish',
				'COM_MENUS_EXTENSION_PUBLISHED_DISABLED',
				'COM_MENUS_HTML_UNPUBLISH_DISABLED',
				'COM_MENUS_EXTENSION_PUBLISHED_DISABLED',
				true,
				'warning',
				'warning'
			),
			-2 => array(
				'publish',
				'COM_MENUS_EXTENSION_UNPUBLISHED_DISABLED',
				'COM_MENUS_HTML_PUBLISH_DISABLED',
				'COM_MENUS_EXTENSION_UNPUBLISHED_DISABLED',
				true,
				'trash',
				'trash'
			),
			-3 => array(
				'publish',
				'',
				'COM_MENUS_HTML_PUBLISH',
				'',
				true,
				'trash',
				'trash'
			),
		);

		return JHtml::_('jgrid.state', $states, $value, $i, 'items.', $enabled, true, $checkbox);
	}
}
