<?php
/**
 * @package    db8optionslite
 *
 * @author     Peter Martin <joomla@db8.eu>
 * @copyright  Copyright 2016-2017 by Peter Martin
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://db8.eu
 */

defined('_JEXEC') or die;

/**
 * Db8optionslite
 *
 * @package  db8optionslite
 * @since    1.0
 */
class Db8optionsliteModelComponents extends JModelList
{

	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'extension_id', 'e.extension_id',
				'active', 'hasparams',
				'name', 'e.name',
				'type', 'e.type', 'type_translated',
				'element', 'e.element',
				'folder', 'e.folder', 'folder_translated',
				'client', 'e.client_id', 'client_translated',
				'params_previous', 'a.params_previous',
				'params_previous_md5', 'a.params_previous_md5',
				'params_imported', 'a.params_imported',
				'params_imported_md5', 'a.params_imported_md5',
			);
		}

		parent::__construct($config);
	}

	protected function populateState($ordering = null, $direction = null)
	{

		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', '', 'string'));
		//$this->setState('filter.type', $this->getUserStateFromRequest($this->context . '.filter.type', 'filter_type', '', 'string'));
		$this->setState('filter.folder', $this->getUserStateFromRequest($this->context . '.filter.folder', 'filter_folder', '', 'string'));
		$this->setState('filter.client_id', $this->getUserStateFromRequest($this->context . '.filter.client_id', 'filter_client_id', null, 'int'));
		$this->setState('filter.hasparams', $this->getUserStateFromRequest($this->context . '.filter.hasparams', 'filter_hasparams', null, 'int'));

		//parent::populateState($ordering, $direction);
		parent::populateState('e.element', 'asc');
	}

	protected function getListQuery()
	{
		// Create a new query object.
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		$query->select(
			$this->getState(
				'list.select', 'DISTINCT a.*, e.extension_id, e.name, e.type, e.element, e.folder, e.client_id, ' .
				'e.enabled, e.access, e.protected, e.manifest_cache, ' .
				'e.params AS params_current, MD5(e.params) AS params_current_md5'
			)
		);
		$query->from('`#__extensions` AS e');
		$query->where($db->quoteName('e.type') . ' = "component"');

		$query->leftJoin('`#__db8optionslite` AS a ON a.extension_key = concat_ws(\'.\', e.name, e.type, e.element, e.folder, e.client_id)');

		$search     = $this->getState('filter.search');
		$active     = $this->getState('filter.active');
		$clientId    = $this->getState('filter.client_id');
		//$type        = $this->getState('filter.type');
		$folder      = $this->getState('filter.folder');
		$extensionId = $this->getState('filter.extension_id');
		$hasparams   = $this->getState('filter.hasparams');

		switch ($active)
		{
			// COM_DB8OPTIONSLITE_FILTER_ACTIVE_NONE
			case '1':
				$query->having($db->quoteName('params_current') . ' = "" ');
				break;

			// COM_DB8OPTIONSLITE_FILTER_ACTIVE_IMPORTED
			case '2':
				$query->having($db->quoteName('params_current_md5') . ' = '. $db->quoteName('a.params_imported_md5'));
				break;

			// COM_DB8OPTIONSLITE_FILTER_ACTIVE_PREVIOUS
			case '3':
				$query->having($db->quoteName('params_current_md5') . ' = '. $db->quoteName('a.params_previous_md5'));
				break;

			// COM_DB8OPTIONSLITE_FILTER_ACTIVE_CHANGED
			// params_current is not empty, and params_current is different from imported & previous settings
			case '4':
				$query->having($db->quoteName('params_current') . ' <> "" AND '
				.  $db->quoteName('params_current_md5') . ' <> '. $db->quoteName('a.params_imported_md5') . ' AND '
				. $db->quoteName('params_current_md5') . ' <> '. $db->quoteName('a.params_previous_md5'));
				break;
		}

		if ($search)
		{
			$query->where($db->quoteName('e.name') . ' LIKE ' . $db->quote('%' . $search . '%'));
		}

		/*if ($type)
		{
			$query->where($db->quoteName('e.type') . ' = ' . $db->quote($type));
		}*/

		if ($clientId != '')
		{
			$query->where($db->quoteName('e.client_id') . ' = ' . (int) $clientId);
		}

		if ($folder != '' && in_array($type, array('plugin', 'library', '')))
		{
			$query->where($db->quoteName('e.folder') . ' = ' . $db->quote($folder == '*' ? '' : $folder));
		}

		if ($extensionId)
		{
			$query->where($db->quoteName('e.extension_id') . ' = ' . $db->quote((int) $extensionId));
		}
		else
		{
			$query->where($db->quoteName('e.extension_id') . ' != ' . $db->quote(0))
				->where($db->quoteName('e.extension_id') . ' != ' . $db->quote(700));
		}

		switch ($hasparams)
		{
			case '3': // all records
				//$query->where($db->quoteName('e.params') . " <> '' ");
				break;
			case '2': // Does not have parameters
				$query->where($db->quoteName('e.params') . " = '' ");
				break;
			default: // Extension has parameters
				$query->where($db->quoteName('e.params') . " <> '' ");
				break;
		}

		$orderCol = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		$query->order($db->escape($orderCol . ' ' . $orderDirn));

		//echo "SQL: ".$query;

		return $query;
	}

    protected function _getList($query, $limitstart = 0, $limit = 0)
    {
        $this->getDbo()->setQuery($query, $limitstart, $limit);
        $result = $this->getDbo()->loadObjectList();

        $this->translate($result);

        return $result;
    }

    protected function translate(&$items)
    {
        $lang = JFactory::getLanguage();

        foreach ($items as &$item)
        {
            $path = $item->client_id ? JPATH_ADMINISTRATOR : JPATH_SITE;

            switch ($item->type)
            {
                case 'component':
                    $extension = $item->element;
                    $source = JPATH_ADMINISTRATOR . '/components/' . $extension;
                    $lang->load("$extension.sys", JPATH_ADMINISTRATOR, null, false, true)
                    ||	$lang->load("$extension.sys", $source, null, false, true);
                    break;
                case 'file':
                    $extension = 'files_' . $item->element;
                    $lang->load("$extension.sys", JPATH_SITE, null, false, true);
                    break;
                case 'library':
                    $extension = 'lib_' . $item->element;
                    $lang->load("$extension.sys", JPATH_SITE, null, false, true);
                    break;
                case 'module':
                    $extension = $item->element;
                    $source = $path . '/modules/' . $extension;
                    $lang->load("$extension.sys", $path, null, false, true)
                    ||	$lang->load("$extension.sys", $source, null, false, true);
                    break;
                case 'plugin':
                    $extension = 'plg_' . $item->folder . '_' . $item->element;
                    $source = JPATH_PLUGINS . '/' . $item->folder . '/' . $item->element;
                    $lang->load("$extension.sys", JPATH_ADMINISTRATOR, null, false, true)
                    ||	$lang->load("$extension.sys", $source, null, false, true);
                    break;
                case 'template':
                    $extension = 'tpl_' . $item->element;
                    $source = $path . '/templates/' . $item->element;
                    $lang->load("$extension.sys", $path, null, false, true)
                    ||	$lang->load("$extension.sys", $source, null, false, true);
                    break;
                case 'package':
                default:
                    $extension = $item->element;
                    $lang->load("$extension.sys", JPATH_SITE, null, false, true);
                    break;
            }

	        $item->client_translated = $item->client_id ? JText::_('JADMINISTRATOR') : JText::_('JSITE');
	        $item->type_translated   = JText::_('COM_DB8OPTIONSLITE_TYPE_' . strtoupper($item->type));
	        $item->folder_translated = @$item->folder ? $item->folder : JText::_('COM_DB8OPTIONSLITE_TYPE_NONAPPLICABLE');

	        // Translate the extension name if possible
            $item->realname = JText::_($item->name);
        }
    }

    public function backup($ids) {

        if ($ids) {

            $return = array();

            foreach ($ids as $id) {
                $query = $this->_db->getQuery(true);
                $query->select("extensions.*, options.extension_key extension_key");
                $query->from("#__extensions as extensions");
                $query->leftJoin("#__db8optionslite as options on concat_ws('.', extensions.name, extensions.type, extensions.element, extensions.folder, extensions.client_id) = options.extension_key");
                $query->where("extensions.extension_id = $id");
                $this->_db->setQuery($query);
                $extension = $this->_db->loadObject();

                $extension_key = $extension->name.".".$extension->type.".".$extension->element.".".$extension->folder.".".$extension->client_id;
                $extension_name = $extension->name;
                $params = addslashes($extension->params);
                $md5 = md5($extension->params);

                if ($extension->extension_key) {
                    $query = $this->_db->getQuery(true);
                    $query->update("#__db8optionslite")
                        ->set(array("params_previous = '$params'", "params_previous_md5 = '$md5'"))
                        ->where("extension_key = '$extension_key'");
                    $this->_db->setQuery($query);
                } else {
                    $query = $this->_db->getQuery(true);
                    $query->insert("#__db8optionslite")
                    ->columns($this->_db->quoteName(array("extension_key", "params_previous", "params_previous_md5")))
                    ->values("'$extension_key', '$params', '$md5'");
                    $this->_db->setQuery($query);
                }
                $this->_db->execute();

                $return[] = $extension_name;
            }

            return $return;

        } else {
            return false;
        }
    }

    public function import($extension_key, $json) {

        $extension_name = strtok($extension_key, '.');

        if ($extension_key) {

            $extension_key = str_replace('.no.', '..', $extension_key);

            $query = $this->_db->getQuery(true);
            $query->select("extensions.*, options.extension_key extension_key");
            $query->from("#__extensions as extensions");
            $query->leftJoin("#__db8optionslite as options on concat_ws('.', extensions.name, extensions.type, extensions.element, extensions.folder, extensions.client_id) = options.extension_key");
            $query->where("concat_ws('.', extensions.name, extensions.type, extensions.element, extensions.folder, extensions.client_id) = '$extension_key'");
            $this->_db->setQuery($query);

            $this->_db->setQuery($query);
            $extension = $this->_db->loadObject();

            if (!$extension)
                return array('status' => 0, 'name' => $extension_name);

            $extension_key = $extension->name.".".$extension->type.".".$extension->element.".".$extension->folder.".".$extension->client_id;
            $extension_name = $extension->name;
            $params = addslashes($extension->params);
            $md5 = md5($extension->params);

            $imported_params = $json;
            $imported_md5 = md5($imported_params);

            if ($extension->extension_key) {
                $query = $this->_db->getQuery(true);
                $query->update("#__db8optionslite")
                    ->set(array("params_previous = '$params'", "params_previous_md5 = '$md5'", "params_imported = '$imported_params'", "params_imported_md5 = '$imported_md5'"))
                    ->where("extension_key = '$extension_key'");
                $this->_db->setQuery($query);
            } else {
                $query = $this->_db->getQuery(true);
                $query->insert("#__db8optionslite")
                    ->columns($this->_db->quoteName(array("extension_key", "params_previous", "params_previous_md5", "params_imported", "params_imported_md5")))
                    ->values("'$extension_key', '$params', '$md5', '$imported_params', '$imported_md5'");
                $this->_db->setQuery($query);
            }
            $this->_db->execute();
            
            return array('status' => 1, 'name' => $extension_name);

        } else {
            return array('status' => 0, 'name' => $extension_name);
        }
    }

}
